#
# Makefile Template for Tessy
# Compiler: GNU Tools for ARM
# Debugger: SEGGER JLink GDB for ARM
# Template Version: 2
#

# Defines to be generated by Tessy:
#
# BSP_DIRECTORYECTORY   :=
# BUFFER_SIZE           :=
# COMPILER_DEFINES      :=
# COMPILER_INCLUDES     :=
# COMPILER_INSTALL_PATH :=
# COMPILER_DEFINES      :=
# COMPILER_INCLUDES     :=
# COMPILER_INSTALL_PATH :=
# ENDIANESS             :=
# EXEFILE_EXTENSION     :=
# LINKER_FILE           :=
# MODULE_PATH           :=
# MODULE_PATH_DOS       :=
# OBJFILE_EXTENSION     :=
# PROJECTROOT           :=
# TARGET_INSTALL_PATH   :=
# TESSY_BIN             :=
# TESSY_SYS             :=
# TESTOBJECT            :=
# TIMER_FILE            :=
# TARGET_INSTALL_PATH   :=
# TIMER_FILE            :=
# USER_LINK_OPTIONS     :=

#####################################################################
#BEGIN_DEFINES
#####################################################################

#
# Special macros for includes and defines
#
TS_DEF_SEPARATOR :=-D
define TS_DEFINE
$(subst ;, $(TS_DEF_SEPARATOR),$(1))
endef

TS_INC_SEPARATOR :=-I
define TS_INCLUDE
$(subst ;, $(TS_INC_SEPARATOR),$(1))
endef

TESSY_SYS_DOS   := $(shell ts_pathname -s $(TESSY_SYS))
TESSY_SYS       := $(shell ts_pathname -s $(TESSY_SYS))
GNU_TOOLS_HOME  := $(shell ts_pathname -s '$(COMPILER_INSTALL_PATH)')
MODULE_PATH_DOS := $(shell ts_pathname -s '$(MODULE_PATH_DOS)')
MODULE_PATH     := $(shell ts_pathname -s '$(MODULE_PATH)')
LINKER_FILE     := $(shell ts_pathname -s '$(LINKER_FILE)')

TS_NAME         := TS_GNUTOOLS
IDE_NAME        := segger
COMPANY_NAME    := gnutools
PROCESSOR_NAME  := arm

ifeq ("little","$(ENDIANESS)")
  TS_BYTEORDER := TS_HOST_BYTEORDER
  TS_BYTEORDER_OPTION :=
endif
ifeq ("big","$(ENDIANESS)")
  TS_BYTEORDER := TS_NET_BYTEORDER
  TS_BYTEORDER_OPTION :=
endif
ifndef TS_BYTEORDER
  $(error Attribute 'Endianess' has invalid value '$(ENDIANESS)', only 'little' and 'big' are allowed.)
endif

all : check_success master slave

# Cygwin work around.
S_BUILDIN_TESSY_FLAG := $(wildcard $(MODULE_PATH)/tslows_builtin.tessy)

check_success :
ifneq ($(S_BUILDIN_TESSY_FLAG),)
	@echo *
	@echo * Obviously the last build failed. Cleaning up built-in files...
	@-ts_rm -f "$(MODULE_PATH)"/tslows_builtin.*
	@echo * Please, try again to generate a built-in target binary.
	@echo *
	@exit 1
endif

#
# MASTER & COMM Makefiles
# (you should not need to modify these settings)
#
M_COMM_MAKE		:=	$(TESSY_SYS)/make/master_$(IDE_NAME)_gnu_i386.mk
S_COMM_MAKE		:=	$(TESSY_SYS)/src/comm/comm_$(IDE_NAME)_$(COMPANY_NAME)_$(PROCESSOR_NAME).mk


#
#
# SLAVE
#
S_CC           := $(shell ts_pathname -s '$(GNU_TOOLS_HOME)')\bin\arm-none-eabi-gcc.exe
S_LINK         := $(shell ts_pathname -s '$(GNU_TOOLS_HOME)')\bin\arm-none-eabi-g++.exe
S_INCLUDES     := -I"$(TESSY_SYS)\include\tessy\comm" -I"$(MODULE_PATH_DOS)"
S_STUB_OBJECT  := $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_stubs$(OBJFILE_EXTENSION)
S_UC_OBJECT    := $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_usr$(OBJFILE_EXTENSION)
# Tested CPU = cortex-m0
# Tested board defines = -DUSE_NUCLEO_F030R8 -DHSI_VALUE=8000000 -DSTM32F0XX_VL -DUSE_STDPERIPH_DRIVER
S_COMP_OPTIONS := -c -mthumb -mcpu=$(CPU) -std=gnu90 $(call TS_DEFINE,$(COMPILER_DEFINES)) -DTESSY -DTS_SLAVE \
                  -D$(TS_NAME) -DTS_HAVE_INT64 -DTS_HAVE_FLOAT64 -O0 -ffunction-sections -fdata-sections -g3 -Wall
S_ASM_OPTIONS  := -c -mthumb -mcpu=$(CPU) -g -Wa,--warn -x assembler-with-cpp
S_LINK_OPTIONS :=  -mcpu=$(CPU) -mthumb -O0 -fmessage-length=0 -fsigned-char -ffunction-sections -fdata-sections -g3 \
                   -Xlinker --gc-sections -specs=nosys.specs -specs=nano.specs

#####################################################################
#END_DEFINES
#####################################################################

#####################################################################
#BEGIN_SOURCE
#####################################################################

#
# Template rules
#
$(MODULE_PATH_DOS)\ts_src$$(SOURCENUMBER)$(OBJFILE_EXTENSION): $(MODULE_PATH_DOS)\ts_src$$(SOURCENUMBER).c
	$(S_CC) $(S_INCLUDES) $(call TS_INCLUDE,$(USER_INCLUDES$$(SOURCENUMBER))) $(S_COMP_OPTIONS) \
		$(call TS_DEFINE,$(USER_DEFINES$$(SOURCENUMBER))$(S_COM_OPTIONS)) \
		$(USER_COMP_OPTIONS$$(SOURCENUMBER)) -o "$@" "$<"


#####################################################################
#END_SOURCE
#####################################################################

#####################################################################
#BEGIN_TARGETS
#####################################################################

ifneq ($(S_GenerateBuiltinData),)
TARGET_FLAG := d
BUILTIN_OBJ := $(MODULE_PATH_DOS)\tslows_builtin$(OBJFILE_EXTENSION)
master :
	@-ts_rm -f "$(MODULE_PATH)"/tslows_builtin$(OBJFILE_EXTENSION)
else
BUILTIN_OBJ :=
TARGET_FLAG := s
master : $(MODULE_PATH)\ts_$(TESTOBJECT)_m.exe
endif
slave  : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_$(TARGET_FLAG)$(EXEFILE_EXTENSION)

# COMPILE USERCODE / STUBS
#
$(S_UC_OBJECT) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_usr.c
	$(S_CC) $(S_INCLUDES) $(S_COMP_OPTIONS) -o "$@" "$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_usr.c"

$(S_STUB_OBJECT) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_stubs.c
	$(S_CC) $(S_INCLUDES) $(S_COMP_OPTIONS) -o "$@" "$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_stubs.c"


#
# COMPILE/LINK MASTER
#

include $(M_COMM_MAKE)


#
# COMPILE SLAVE COMMUNICATION MODULES

include $(S_COMM_MAKE)


#
#
# COMPILE/LINK SLAVE
#
$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(OBJFILE_EXTENSION) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s.c
	@echo *
	@echo ***** Compiling Slave *****
	$(S_CC) $(S_COMP_OPTIONS) $(S_INCLUDES) -o "$@" "$<"

#
# Handle Board Files
#
ifeq ("true","$(USE_BOARD_FILES)")
ifeq ("","$(INITOBJDIR)")
$(error Attribute 'Use Board Files' is set to true but 'InitObjDir' is empty.)
endif
INITOBJDIR := $(shell ts_pathname -s '$(INITOBJDIR)')
ifeq ("","$(INITSRCDIR)")
$(info Attribute 'InitSrcDir' is empty. Using objects for linking only.)
ARM_OBJS   := $(wildcard $(INITOBJDIR)/*$(OBJFILE_EXTENSION))
else
INITSRCDIR := $(shell ts_pathname -s '$(INITSRCDIR)')
ARM_OBJS   := $(patsubst $(INITSRCDIR)/%.c,$(INITOBJDIR)/%$(OBJFILE_EXTENSION),$(wildcard $(INITSRCDIR)/*.c)) \
              $(patsubst $(INITSRCDIR)/%.s,$(INITOBJDIR)/%$(OBJFILE_EXTENSION),$(wildcard $(INITSRCDIR)/*.s))

$(INITOBJDIR)/%$(OBJFILE_EXTENSION) : $(INITSRCDIR)/%.c
	$(S_CC) $(S_COMP_OPTIONS) $(S_INCLUDES) -I$(INITSRCDIR) -c "$<" -o "$@"

$(INITOBJDIR)/%$(OBJFILE_EXTENSION) : $(INITSRCDIR)/%.s
	$(S_CC) $(S_COMP_OPTIONS) $(S_INCLUDES) -c "$<" -o "$@"
endif
ifeq ("","$(ARM_OBJS)")
$(warning Attribute 'Use Board Files' is set to true but no objects were found.)
endif
endif


$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_$(TARGET_FLAG)$(EXEFILE_EXTENSION) :  $(BUILTIN_OBJ) $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(OBJFILE_EXTENSION) \
															$(S_SRC_OBJECTS) $(ARM_OBJS) \
															$(S_UC_OBJECT) \
															$(S_STUB_OBJECT) \
															$(S_OBJECTS)
	@echo *
	@echo ***** Linking Slave *****
	$(S_LINK) $(S_LINK_OPTIONS) $(ARM_OBJS) \
				 $(S_OBJECTS) $(BUILTIN_OBJ) $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(OBJFILE_EXTENSION) \
				 $(S_SRC_OBJECTS) $(S_UC_OBJECT) $(S_STUB_OBJECT) \
				 $(USER_LINK_OPTIONS) $(LIBRARY) -T $(LINKER_FILE) \
				 -Wl,-Map="$(MODULE_PATH_DOS)\ts_$(TESTOBJECT).map" \
				 -o "$@"
ifneq ($(S_GenerateBuiltinData),)
	@-ts_rm -f "$(MODULE_PATH)"/tslows_builtin.*
	@echo *
	@echo * Generated target binary '$(shell ts_pathname -w $(MODULE_PATH)\ts_$(TESTOBJECT)_$(TARGET_FLAG)$(EXEFILE_EXTENSION))' containing builtin data.
	@echo *
endif	
	

#####################################################################
#END_TARGETS
#####################################################################

